#include <math.h> 
#include <vector> 
#include <iostream>
#include <string.h>
#include <stdlib.h>

double HeavisideTheta(double x)
{
  return (x >= 0.0) ? 1.0 : 0.0;
}

double Simpson_Integration(const std::vector<double>& x, const std::vector<double>& y)
{
  if (x.size() != y.size())
    {
      std::cerr << "Error: x and y vectors must have the same length." << std::endl;
      return 0.0;
    }

  if (x.size() < 3 )
    {
      std::cerr << "Error: Simpson's rule requires an odd number of data points." << std::endl;
      return 0.0;
    }

  double sum = 0.0;
  for (size_t i = 0; i < x.size() - 3; i += 2) {
    double h = (x[i + 2] - x[i]) / 2.0;
    sum += (h / 3.0) * (y[i] + 4 * y[i + 1] + y[i + 2]);
  }

  return sum;
}


double trapezoidal_Integration(const std::vector<double>& x, const std::vector<double>& y)
{
  if (x.size() != y.size())
    {
      std::cerr << "Error: x and y vectors must have the same length." << std::endl;
      return 0.0;
    }

  double sum = 0.0;
  if(x.size()>2)
    {
      for (size_t i = 0; i < x.size() - 1; ++i)
	{
	  double h = x[i + 1] - x[i];  // Width of the interval
	  sum += 0.5 * h * (y[i] + y[i + 1]);  // Area of each trapezoid
	}
    }
  else
    {
      sum = 0.0;
    }

  return sum;
}



double VelocitySpectrum(double Vchi, double theta, double VE, double Vesc, double V0)
{
    // HeavisideTheta[Vchi]
    double heavisideVchi = HeavisideTheta(Vchi);

    // Constants
    const double sqrtPi = sqrt(M_PI);
    
    // Calculate normalization factor
    double normalization = pow(V0, 3) * (sqrtPi *erf(Vesc / V0) - 
                         2.0 * (Vesc / V0) * exp(-(pow(Vesc, 2) /pow(V0, 2))));

    // Exp[-((Vchi^2 + VE^2 + 2 * Vchi * VE * cos(theta)) / V0^2)]
    double exponent = -(pow(Vchi, 2) +pow(VE, 2) + 2.0 * Vchi * VE *cos(theta)) /pow(V0, 2);
    double expTerm =exp(exponent);

    // HeavisideTheta[Vesc - Sqrt[Vchi^2 + VE^2 + 2 * Vchi * VE * cos(theta)]]
    double velocityMagnitude =sqrt(pow(Vchi, 2) +pow(VE, 2) + 2.0 * Vchi * VE *cos(theta));
    double heavisideVesc = HeavisideTheta(Vesc - velocityMagnitude);

    // Final result
    return heavisideVchi * (1.0 / (M_PI * normalization)) * expTerm * heavisideVesc;
}

double VeloSpecInt(double Vchi, double VE, double Vesc, double V0, int N)
{
  double theta_min = 0.0;
  double theta_max = M_PI;
  double dtheta = (theta_max - theta_min) / N;  // Step size
  double integral = 0.0;

  for (int i = 0; i <= N; ++i)
    {
      double theta = theta_min + i * dtheta;
      double weight = (i == 0 || i == N) ? 0.5 : 1.0;  // Trapezoidal rule weights
      integral += weight * 2 * M_PI *pow(Vchi, 3) *sin(theta) * VelocitySpectrum(Vchi, theta, VE, Vesc, V0);
    }

  integral *= dtheta;  // Multiply by step size
  return integral;
}
